endpoint_stop = "/kodiak/v1/device/capture/stop"
endpoint_start = "/kodiak/v1/device/capture/start"

endpoint_capture_status = "/kodiak/v1/device/capture/status"
endpoint_status = "/kodiak/v1/status"


endpoint_users = "/kodiak/v1/users"

# endpoint_trigger_status = "/kodiak/v1/device/capture/trigger/status"

endpoint_login = "/kodiak/v1/login"
endpoint_refresh_login = "/kodiak/v1/login/refresh"

endpoint_lock = "/kodiak/v1/device/lock"
endpoint_unlock = "/kodiak/v1/device/unlock"

endpoint_lock_status = "/kodiak/v1/device/lock"
