class KodiakGlobals:
    url_prefix = 'https://'
    last_response = []
    lock_name = "sanblaze_lock"
    database_name = "sanblaze_db"

    my_kodiak_ip = "192.168.1.247"
    my_kodiak_user = "mholsey"
    # my_kodiak_user = "sanblaze"
    my_kodiak_pass = "sb"

    kodiak_mdns_locator = '_g5_busxpert._tcp.local.'
